#region usings
using System;
using System.ComponentModel.Composition;
using System.Text;
using System.IO;
using System.Resources;
using System.Reflection;
using System.Collections;
using System.Collections.Generic;

using VVVV.PluginInterfaces.V1;
using VVVV.PluginInterfaces.V2;
using VVVV.Utils.VColor;
using VVVV.Utils.VMath;

using VVVV.Core.Logging;

using com.drew.metadata;
using com.drew.metadata.iptc;
using com.drew.imaging.jpg;
using com.drew.imaging.tiff;



#endregion usings

namespace VVVV.Nodes
{
	#region PluginInfo
	[PluginInfo(Name = "MetaData", Category = "File", Help = "Extracts EXIF & other MetaData", Tags = "EXIF, Metadata", Author="Zeos, Ferret Renaud, Drew Noakes, Andreas Huggel, Phil Harvey")]
	#endregion PluginInfo
	public class FileMetaDataNode : IPluginEvaluate
	{
		#region fields & pins
		[Input("Input")]
		IDiffSpread<string> FInput;
		
		[Input("Update", IsBang=true)]
		IDiffSpread<bool> FUpdate;

		[Output("Tag")]
		ISpread<string> FOutput;
		
		[Output("Description")]
		ISpread<string>FDescription;
		
		[Output("Bins")]
		ISpread<int>FBins;
		
		[Output("Status")]
		ISpread<bool>FStatus;

		[Import()]
		ILogger FLogger;
		#endregion fields & pins
		
		
		//called when data for any output pin is requested
		public void Evaluate(int SpreadMax)
		{
			if(FInput.IsChanged || FUpdate.IsChanged) 
			{
				FOutput.SliceCount = 0;
				FDescription.SliceCount = 0;
				FBins.SliceCount = FInput.SliceCount;
				FStatus.SliceCount = 0;
				for(int i =0; i<FInput.SliceCount; i++)
				{
					Metadata lcMetadata = null;
					
					try 
					{
						FileInfo lcImgFile = new FileInfo(FInput[i]);
						if(!FInput[i].ToLower().EndsWith(".jpg"))
							lcMetadata = TiffMetadataReader.ReadMetadata(lcImgFile);
						else
	            			lcMetadata = JpegMetadataReader.ReadMetadata(lcImgFile);
		       		
		        	} 
					catch (JpegProcessingException e) 
					{
		            	FLogger.Log(LogType.Debug, "error 1a: "+ e.Message);
		        	} 
					catch (IOException e) 
					{
		            	FLogger.Log(LogType.Debug, "error 1b: " + e.Message);
		        	}
					
					FBins[i] = 0;
					//extract
					if(lcMetadata != null) 
					{
						
						foreach(AbstractDirectory lcDirectory in lcMetadata)
		           	 	{
			                // We look for potential error
			                if (lcDirectory.HasError)
			                {
			                    FLogger.Log(LogType.Debug, "Some errors were found, activate trace using /d:TRACE option with the compiler");
			                }
			                // Then we want all tags, so we iterate on the current directory
			                foreach(Tag lcTag in lcDirectory) 
		           	 		{
			                    string lcTagDescription = "";
			                    try
			                    {
			                        lcTagDescription = lcTag.GetDescription();
			                    }
			                    catch (MetadataException e)
			                    {
			                        FLogger.Log(LogType.Debug,e.Message);
			                    }
			                    string lcTagName = lcTag.GetTagName();
			                	FOutput.Add(lcTagName);
			                	FDescription.Add(lcTagDescription);
			                   	FBins[i]++;
			                    lcTagDescription = null;
			                    lcTagName = null;
			                }
			            }
		            	FStatus.Add(true);
						
						lcMetadata = null;
					} 
					else 
					{
						FStatus.Add(false);
						
					}	
					
				}
			}

		}

			//}
	}
}
